--[[
返回table的元素个数
--]]
function table.nums(t)
    local count = 0
    for k, v in pairs(t) do
        count = count + 1
    end
    return count
end

--[[
返回table的全部KEY
--]]
function table.keys(t)
    local keys = {}
	if type(t) == "table" then
		for k, v in pairs(t) do
			keys[#keys + 1] = k
		end
	end
    return keys
end

--[[
返回table的全部values
--]]
function table.values(t)
    local values = {}
    for k, v in pairs(t) do
        values[#values + 1] = v
    end
    return values
end

--[[
把两个table合并
dest:(table) 被合并的table
src:(table)
注意：KEY相同时，dest中的会被替换
--]]
function table.merge(dest, src)
    for k, v in pairs(src) do
        dest[k] = v
    end
end

--[[
复制一个新的table
--]]
function table.copy(t)  
    local tab = {}  
    for k, v in pairs(t or {}) do  
        if type(v) ~= "table" then  
            tab[k] = v  
        else  
            tab[k] = table.copy(v)  
        end  
    end  
    return tab  
end 


--[[

insert list.

**Usage:**

    local dest = {1, 2, 3}
    local src  = {4, 5, 6}
    table.insertTo(dest, src)
    -- dest = {1, 2, 3, 4, 5, 6}
	dest = {1, 2, 3}
	table.insertTo(dest, src, 5)
    -- dest = {1, 2, 3, nil, 4, 5, 6}


@param table dest
@param table src
@param table begin insert position for dest
表内追加数据
--]]
function table.insertTo(dest, src, begin)
	begin = tonumber(begin)
	if begin == nil then
		begin = #dest + 1
	end

	local len = #src
	for i = 0, len - 1 do
		dest[i + begin] = src[i + 1]
	end
end
--[[
找target在list中的索引下标,
最后两个参数可以为空
search target index at list.

@param table list
@param * target
@param int from idx, default 1
@param bool useNaxN, the len use table.maxn(true) or #(false) default:false
@param return index of target at list, if not return -1
]]
function table.indexOf(list, target, from, useMaxN)
	local len = (useMaxN and #list) or table.maxn(list)
	if from == nil then
		from = 1
	end
	for i = from, len do
		if list[i] == target then
			return i
		end
	end
	return -1
end

--[[
找列表中，对应KEY、VALUE的下标
--]]
function table.indexOfKey(list, key, value, from, useMaxN)
	local len = (useMaxN and #list) or table.maxn(list)
	if from == nil then
		from = 1
	end
	local item = nil
	for i = from, len do
		item = list[i]
		if item ~= nil and item[key] == value then
			return i
		end
	end
	return -1
end
--[[
删除列表中的item，如果是removeAll=true，
会把所有对应的item删除
--]]
function table.removeItem(t, item, removeAll)
    for i = #t, 1, -1 do
        if t[i] == item then
            table.remove(t, i)
            if not removeAll then break end
        end
    end
end
--[[
对每个v,k调用fun
并把返回的结果，给列表t[k]赋值
fun的参数顺序为fun(v, k)
--]]
function table.map(t, fun)
    for k,v in pairs(t) do
        t[k] = fun(v, k)
    end
end
--[[
对每个v,k调用fun
fun的参数顺序为fun(v, k)
--]]
function table.walk(t, fun)
    for k,v in pairs(t) do
        fun(v, k)
    end
end
--[[
用于过滤元素，
列表中对应的k、v，在fun下返回false
则该元素在列表中被删去
--]]
function table.filter(t, fun)
    for k,v in pairs(t) do
        if not fun(v, k) then
            t[k] = nil
        end
    end
end

--[[
在t表中，查找是否存在item
有的返回true
--]]
function table.find(t, item)
    return table.keyOfItem(t, item) ~= nil
end
--[[
返回一个去掉相同元素的table
--]]
function table.unique(t)
    local r = {}
    local n = {}
    for i = #t, 1, -1 do
        local v = t[i]
        if not r[v] then
            r[v] = true
            n[#n + 1] = v
        end
    end
    return n
end
--[[
在t表中，查找是否存在item
有的返回item对应的key
--]]
function table.keyOfItem(t, item)
    for k,v in pairs(t) do
        if v == item then return k end
    end
    return nil
end

--[[
针对下标有缺失，需要重新排列的
--]]
function table.reSortIndex(t)
	local newTable = {}
	for _,v in pairs(t) do	
		table.insert(newTable, v)
	end
	return newTable
end

function table.toString(_t)  
    local delm = " ";
    local szRet = "{\n"..delm  
    local function doT2S(_i, _v)  
    delm = delm.." "
        if "number" == type(_i) then  
            szRet = szRet .. "[" .. _i .. "] = "  
            if "number" == type(_v) then  
                szRet = szRet .. _v .. ",\n"..delm   
            elseif "string" == type(_v) then  
                szRet = szRet .. '"' .. _v .. '"' .. ",\n"..delm 
            elseif "table" == type(_v) then  
                szRet = szRet .. table.toString(_v) .. ",\n"..delm 
            else  
                szRet = szRet .. "nil,\n"..delm 
            end  
        elseif "string" == type(_i) then  
            szRet = szRet .. '["' .. _i .. '"] = '  
            if "number" == type(_v) then  
                szRet = szRet .. _v .. ",\n"..delm 
            elseif "string" == type(_v) then  
                szRet = szRet .. '"' .. _v .. '"' .. ",\n"..delm 
            elseif "table" == type(_v) then  
                szRet = szRet .. table.toString(_v) .. ",\n"..delm 
            else  
                szRet = szRet .. "nil,\n"..delm 
            end  
        end  
    end  
    table.foreach(_t, doT2S)  
    szRet = szRet .. "\n}"  
    return szRet 
end